"""
A library that provides a python interface to Bitkub API
"""
import hashlib
import hmac
import json
import time

from .constants import ENDPOINTS
from .decorators import check_in_attributes
from .request import basic_request


class Bitkub:
    def __init__(self, api_key=None, api_secret=None):
        self.api_key = api_key
        self.api_secret = api_secret
        self.API_ROOT = ENDPOINTS["API_ROOT"]

    def _get_api_secret(self):
        return self.api_secret.encode()

    def _get_path(self, path_name, **kwargs):
        """
        Get full endpoint for a specific path.
        """
        return self.API_ROOT + ENDPOINTS[path_name].format(**kwargs)

    def _json_encode(self, payload):
        return json.dumps(payload, separators=(',', ':'), sort_keys=True)

    def _get_headers(self):
        headers = {
            "ACCEPT": "application/json",
            "CONTENT-TYPE": "application/json",
            "X-BTK-APIKEY": "{0}".format(self.api_key)
        }

        return headers

    def _get_signature(self, payload):
        message = self._json_encode(payload)
        signature = hmac.new(self._get_api_secret(), msg=message.encode(), digestmod=hashlib.sha256).hexdigest()

        return signature

    def _get_timestamp(self):
        timestamp = int(time.time())

        return timestamp

    def _get_payload(self, **kwargs):
        payload = {"ts": self._get_timestamp()}
        payload.update(kwargs)
        payload["sig"] = self._get_signature(payload)
        payload = self._json_encode(payload)

        return payload

    def set_api_key(self, api_key):
        self.api_key = api_key

    def set_api_secret(self, api_secret):
        self.api_secret = api_secret

    def status(self):
        url = self._get_path("STATUS_PATH")

        return basic_request('GET', url)

    def servertime(self):
        url = self._get_path("SERVERTIME_PATH")

        return basic_request('GET', url)

    def symbols(self):
        url = self._get_path("MARKET_SYMBOLS_PATH")

        return basic_request('GET', url)

    def ticker(self, sym=''):
        url = self._get_path("MARKET_TICKER_PATH", sym=sym)

        return basic_request('GET', url)

    def trades(self, sym='', lmt=1):
        url = self._get_path("MARKET_TRADES_PATH", sym=sym, lmt=lmt)

        return basic_request('GET', url)

    def bids(self, sym='', lmt=1):
        url = self._get_path("MARKET_BIDS_PATH", sym=sym, lmt=lmt)

        return basic_request('GET', url)

    def asks(self, sym='', lmt=1):
        url = self._get_path("MARKET_ASKS_PATH", sym=sym, lmt=lmt)

        return basic_request('GET', url)

    def books(self, sym='', lmt=1):
        url = self._get_path("MARKET_BOOKS_PATH", sym=sym, lmt=lmt)

        return basic_request('GET', url)

    def tradingview(self, sym='', int=1, frm='', to=''):
        url = self._get_path("MARKET_TRADING_VIEW_PATH", sym=sym, int=int, frm=frm, to=to)

        return basic_request('GET', url)

    def depth(self, sym='', lmt=1):
        url = self._get_path("MARKET_DEPTH_PATH", sym=sym, lmt=lmt)

        return basic_request('GET', url)

    @check_in_attributes(["api_key", "api_secret"])
    def wallet(self):
        url = self._get_path("MARKET_WALLET")
        payload = self._get_payload()

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def balances(self):
        url = self._get_path("MARKET_BALANCES")
        payload = self._get_payload()

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def place_bid(self, sym='', amt=1, rat=1, typ='limit', client_id=''):
        url = self._get_path("MARKET_V2_PLACE_BID")
        payload = self._get_payload(sym=sym, amt=amt, rat=rat, typ=typ, client_id=client_id)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_key"])
    def place_bid_test(self, sym='', amt=1, rat=1, typ='limit', client_id=''):
        url = self._get_path("MARKET_PLACE_BID_TEST")
        payload = self._get_payload(sym=sym, amt=amt, rat=rat, typ=typ, client_id=client_id)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def place_ask(self, sym='', amt=1, rat=1, typ='limit', client_id=''):
        url = self._get_path("MARKET_V2_PLACE_ASK")
        payload = self._get_payload(sym=sym, amt=amt, rat=rat, typ=typ, client_id=client_id)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def place_ask_test(self, sym='', amt=1, rat=1, typ='limit', client_id=''):
        url = self._get_path("MARKET_PLACE_ASK_TEST")
        payload = self._get_payload(sym=sym, amt=amt, rat=rat, typ=typ, client_id=client_id)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    # 2023-03-27 Deprecated
    @check_in_attributes(["api_key", "api_secret"])
    def place_ask_by_fiat(self, sym='', amt=1, rat=1, typ='limit', client_id=''):
        coin_amt = amt/rat if rat > 0 else amt
        return self.place_ask(sym=sym, amt=coin_amt, rat=rat, typ=typ, client_id=client_id)

    @check_in_attributes(["api_key", "api_secret"])
    def cancel_order(self, sym='', id='', sd='buy', hash=''):
        url = self._get_path("MARKET_V2_CANCEL_ORDER")
        payload = self._get_payload(sym=sym, id=id, sd=sd, hash=hash)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def my_open_orders(self, sym=''):
        url = self._get_path("MARKET_MY_OPEN_ORDERS")
        payload = self._get_payload(sym=sym)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def my_open_history(self, sym='', p=1, lmt=10, start=None, end=None):
        url = self._get_path("MARKET_MY_ORDER_HISTORY")
        payload = self._get_payload(sym=sym, p=p, lmt=lmt, start=start, end=end)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def order_info(self, sym='', id=None, sd='buy', hash=''):
        url = self._get_path("MARKET_ORDER_INFO")
        payload = self._get_payload(sym=sym, id=id, sd=sd, hash=hash)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def crypto_address(self, p=1, lmt=10):
        url = self._get_path("CRYPTO_ADDRESSES")
        payload = self._get_payload(p=p, lmt=lmt)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def crypto_withdraw(self, cur='', amt=0, adr='', mem=''):
        url = self._get_path("CRYPTO_WITHDRAW")
        payload = self._get_payload(cur=cur, amt=amt, adr=adr, mem=mem)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def crypto_internal_withdraw(self, cur='', amt=0, adr='', mem=''):
        url = self._get_path("CRYPTO_INTERNAL_WITHDRAW")
        payload = self._get_payload(cur=cur, amt=amt, adr=adr, mem=mem)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def crypto_deposit_history(self, p=1, lmt=10):
        url = self._get_path("CRYPTO_DEPOSIT_HISTORY")
        payload = self._get_payload(p=p, lmt=lmt)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def crypto_withdraw_history(self, p=1, lmt=10):
        url = self._get_path("CRYPTO_WITHDRAW_HISTORY")
        payload = self._get_payload(p=p, lmt=lmt)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def crypto_generate_address(self, sym=''):
        url = self._get_path("CRYPTO_GENERATE_ADDRESS")
        payload = self._get_payload(sym=sym)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def fiat_accounts(self, p=1, lmt=10):
        url = self._get_path("FIAT_ACCOUNTS")
        payload = self._get_payload(p=p, lmt=lmt)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def fiat_withdraw(self, id='', amt=0):
        url = self._get_path("FIAT_WITHDRAW")
        payload = self._get_payload(id=id, amt=amt)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def fiat_deposit_history(self, p=1, lmt=10):
        url = self._get_path("FIAT_DEPOSIT_HISTORY")
        payload = self._get_payload(p=p, lmt=lmt)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def fiat_withdraw_history(self, p=1, lmt=10):
        url = self._get_path("FIAT_WITHDRAW_HISTORY")
        payload = self._get_payload(p=p, lmt=lmt)

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def market_wstoken(self):
        url = self._get_path("MARKET_WSTOKEN")
        payload = self._get_payload()

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def user_limits(self):
        url = self._get_path("USER_LIMITS")
        payload = self._get_payload()

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)

    @check_in_attributes(["api_key", "api_secret"])
    def user_trading_credits(self):
        url = self._get_path("USER_TRADING_CREDITS")
        payload = self._get_payload()

        return basic_request('POST', url, headers=self._get_headers(), payload=payload)



from bitkub import Bitkub
from unittest import TestCase


class BitkubTest(TestCase):

    @classmethod
    def setUpClass(cls):
        API_KEY = "XXXXXX"
        API_SECRET = "XXXXXX"

        cls.bitkub = Bitkub()
        cls.bitkub.set_api_key(API_KEY)
        cls.bitkub.set_api_secret(API_SECRET)

    def test_status(self):
        # print(self.bitkub.status())
        self.assertTrue(True)

    def test_servertime(self):
        # print(self.bitkub.servertime())
        self.assertTrue(True)

    def test_symbols(self):
        # print(self.bitkub.symbols())
        self.assertTrue(True)

    def test_ticker(self):
        # print(self.bitkub.ticker('THB_BTC'))
        self.assertTrue(True)

    def test_trades(self):
        # print(self.bitkub.trades(sym="THB_BTC", lmt=2))
        self.assertTrue(True)

    def test_bids(self):
        # print(self.bitkub.bids(sym="THB_BTC", lmt=2))
        self.assertTrue(True)

    def test_asks(self):
        # print(self.bitkub.asks(sym="THB_BTC", lmt=2))
        self.assertTrue(True)

    def test_books(self):
        # print(self.bitkub.books(sym="THB_BTC", lmt=1))
        self.assertTrue(True)

    def test_tradingview(self):
        # print(self.bitkub.tradingview())
        self.assertTrue(True)

    def test_depth(self):
        # print(self.bitkub.depth(sym='THB_BTC', lmt=1))
        self.assertTrue(True)

    def test_wallet(self):
        # print(self.bitkub.wallet())
        self.assertTrue(True)

    def test_balances(self):
        # print(self.bitkub.balances())
        self.assertTrue(True)

    def test_place_bid(self):
        # print(self.bitkub.place_bid(sym='THB_BTC'))
        self.assertTrue(True)

    def test_place_ask(self):
        # print(self.bitkub.place_ask(sym='THB_BTC'))
        self.assertTrue(True)

    # 2023-03-27 Deprecated
    # def test_place_ask_by_fiat(self):
    #     # print(self.bitkub.place_ask_by_fiat(sym='THB_BTC'))
    #     self.assertTrue(True)

    def test_cancel_order(self):
        # print(self.bitkub.cancel_order(sym='THB_BTC', id=1, sd='buy', hash='123'))
        self.assertTrue(True)

    def test_my_open_orders(self):
        # print(self.bitkub.my_open_orders(sym='THB_BTC'))
        self.assertTrue(True)

    def test_my_open_history(self):
        # print(self.bitkub.my_open_history(sym='THB_BTC'))
        self.assertTrue(True)

    def test_order_info(self):
        # print(self.bitkub.order_info(sym='THB_BTC'))
        self.assertTrue(True)

    def test_crypto_address(self):
        # print(self.bitkub.crypto_address())
        self.assertTrue(True)

    def test_crypto_withdraw(self):
        # print(self.bitkub.crypto_withdraw())
        self.assertTrue(True)

    def test_crypto_deposit_history(self):
        # print(self.bitkub.crypto_deposit_history())
        self.assertTrue(True)

    def test_crypto_withdraw_history(self):
        # print(self.bitkub.crypto_withdraw_history())
        self.assertTrue(True)

    def test_fiat_accounts(self):
        # print(self.bitkub.fiat_accounts())
        self.assertTrue(True)

    def test_fiat_withdraw(self):
        # print(self.bitkub.fiat_withdraw(id='123', amt=1))
        self.assertTrue(True)

    def test_fiat_deposit_history(self):
        # print(self.bitkub.fiat_deposit_history())
        self.assertTrue(True)

    def test_fiat_withdraw_history(self):
        # print(self.bitkub.fiat_withdraw_history())
        self.assertTrue(True)

    def test_market_wstoken(self):
        # print(self.bitkub.market_wstoken())
        self.assertTrue(True)

    def test_user_limits(self):
        # print(self.bitkub.user_limits())
        self.assertTrue(True)

    def test_user_trading_credits(self):
        # print(self.bitkub.user_trading_credits())
        self.assertTrue(True)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='bitkub-v2',
    version='2.1.0',
    description='A Python library for Bitkub API v2',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/appcorner/bitkub',
    author='appcorner',
    author_email='appcorner@yahoo.com',
    license='MIT',
    scripts=[],
    keywords=['bitkub', 'bitkub-python', 'bitkub-python-sdk'],
    packages=['bitkub-v2'],
    install_requires=['requests'],
)
